import os
from flask import Flask
from config import Config
from flask_security import Security, SQLAlchemyUserDatastore, hash_password
from models_shared import db, User, Role

from flask_wtf.csrf import CSRFProtect
from flask_talisman import Talisman

# Create app
app = Flask(__name__)
app.config.from_object(Config)

app.template_folder = '../frontend/build'
app.static_folder = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'frontend', 'build', 'static'))
app.build_folder = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'frontend', 'build'))


# Define security configurations:
if not app.config['DEBUG']:
    exit("App is not secure, security configurations for production environment must be made")

# Register the blueprints with the app
from routes_default import default as default_blueprint
app.register_blueprint(default_blueprint)

from routes_auth import auth as auth_blueprint
app.register_blueprint(auth_blueprint)

# Enable CSRF on all api endpoints.
csrf = CSRFProtect(app)

# Define Content Security Policy where all content comes from the site's own origin
csp = {
    'default-src': '\'self\''
}
talisman = Talisman(app, content_security_policy=csp)

# Initialize Flask-Security
user_datastore = SQLAlchemyUserDatastore(db, User, Role)
app.security = Security(app, user_datastore)


if __name__ == '__main__':
   
    with app.app_context():
        # Bind the database to the app
        db.init_app(app)
        db.create_all()
        
        if not app.security.datastore.find_user(email="test@me.com"):
            app.security.datastore.create_user(
                email="test@me.com",
                username="test",
                active=False,
                password=hash_password("test")
            )
        db.session.commit()   

        # Start the app
        app.run()