from flask_sqlalchemy import SQLAlchemy
from sqlalchemy import ForeignKey
from sqlalchemy.orm import Mapped, mapped_column, relationship
from sqlalchemy.types import Integer, String, Boolean
from typing import List


db = SQLAlchemy()

class User(db.Model):
    __tablename__ = "user_table"

    def get_id(self) -> str:
        """(Imported from Flask-Security/Flask-Login)
           This method must return a str that uniquely identifies this user,
           and can be used to load the user from the user_loader callback.'
           Note that this must be a str - if the ID is natively an int or some other type,
           you will need to convert it to str..
        """
        return self.fs_uniquifier


    @property
    def is_authenticated(self):
        """(Imported from Flask-Login)
           This property should return True if the user is authenticated"""
        return self.authenticated

    @property
    def is_active(self) -> bool:
        """(Imported from Flask-Security/Flask-Login)
           Returns `True` if the user is active, in addition to being authenticated"""
        return self.active

    @property
    def is_anonymous(self):
        """(Imported from Flask-Login)
        Actual users should return False"""
        return False


    id: Mapped[int] = mapped_column(Integer, primary_key=True)
    email: Mapped[str] = mapped_column(String(255), unique=True, nullable=True)
    username: Mapped[str] = mapped_column(String(80), unique=True, nullable=False)
    password: Mapped[str] = mapped_column(String(255), nullable=True)
    authenticated: Mapped[bool] = mapped_column(Boolean, default=False, nullable=True) 
    active: Mapped[bool] = mapped_column(Boolean, default=True, nullable=False) 
    

    # Flask-Security user identifier
    fs_uniquifier: Mapped[str] = mapped_column(String(64), unique=True, nullable=False)
    roles: Mapped[List["Role"]] = relationship()


class Role(db.Model):
    __tablename__  = "role_table"
    role_id: Mapped[int] = mapped_column(Integer, primary_key=True)
    name: Mapped[str] = mapped_column(String(80), nullable=False)
    description: Mapped[str] = mapped_column(String())
    parent_id: Mapped[int] = mapped_column(ForeignKey("user_table.id"))

    