import re
from flask import Blueprint
from flask import render_template, jsonify
from flask import current_app, request 
from flask_security import hash_password, login_user, verify_password


default = Blueprint("default", __name__)

import email_validator
email_validator.SPECIAL_USE_DOMAIN_NAMES.remove("test")


@default.route("/", methods=["GET", "POST"])
def register():

    if request.method == 'GET':
        return render_template('index.html')

    if request.method == 'POST':
        data = request.get_json() 
        
        _validated_email = None
        _validated_username = None
        _validated_password = None

        _data_faults = {"info": "faults",
                        "fault_email" : None,
                        "fault_username" : None,
                        "fault_password" : None}

        # Validate email
        try:
             _validated_email = current_app.security._mail_util.validate(data["email"])

        except ValueError as e:
            _data_faults["fault_email"] = True    

        
        # Validate username
        regex__name_pattern = "^[a-zA-Z0-9]{3,20}" 
        match_name = re.fullmatch(regex__name_pattern, data["username"])
        if match_name: 
            ubad, unorm = current_app.security._username_util.validate(data["username"])
            if ubad is None: 
                _validated_username = unorm
            else:
                _data_faults["fault_username"] = ubad
        
        # Validate password
        regex_passw_pattern = "^(?=.*[a-zA-Z])(?=.*[0-9])[A-Za-z0-9\w\W]{4,}$" 
        match_passwd = re.fullmatch(regex_passw_pattern, data["password"]) 
        if match_passwd:
            pbad, pnorm = current_app.security._password_util.validate(data["password"], False)
            if pbad is None:
                _validated_password = pnorm
            else:
                _data_faults["fault_password"] = pbad
        
                
        if _validated_email and _validated_username and _validated_password:
            # Check for existing user with that email or name
            check_existing_email = current_app.security.datastore.find_user(email=_validated_email)
            check_existing_username = current_app.security.datastore.find_user(username=_validated_username)    
            if check_existing_email is None and check_existing_username is None:
                # Create a new user
                user = current_app.security.datastore.create_user(
                        active = True,
                        authenticated = True,
                        email = _validated_email,
                        username = _validated_username,
                        password = hash_password(_validated_password))
                current_app.security.datastore.commit()


                # This will login the user and store a remember me cookie on the clients machine.
                login_user(user, remember=False)
                
                
                user = current_app.security.datastore.find_user(username=_validated_username)   
                print() 
                print("User created info:")
                print(user.id)
                print(user.email)
                print(user.username)
                print(user.password)
                print(user.authenticated)
                print(user.active)  
                print(user.fs_uniquifier)
                print()

                return jsonify(success=True)
                
            else: 
                # User already exists
                data_reg_faults = {"info": "reg-faults"}
                if check_existing_email is not None:
                    data["fault_ext_email"] = True
                if check_existing_username is not None:
                    data["fault_ext_username"] = True
                return jsonify(data_reg_faults)
       
        else: 
            # Faults with the supplied information from the user
            print("FAULTS")
            return jsonify(_data_faults)


@default.route("/login", methods=["POST"])
def login():
    if request.method == 'POST':
        data = request.get_json()
        unorm = current_app.security._username_util.normalize(data["username"])
        user = current_app.security.datastore.find_user(username=unorm)
        if user is not None: 
            pnorm = current_app.security._password_util.normalize(data["password"])

            # Compare the supplied hash with the stored hash for the user
            if verify_password(pnorm, user.password):
                login_user(user, remember=False)
                return jsonify(success=True)
            else: 
                return jsonify(success=False, message="Invalid username or password")
    
 